# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/fingerprint.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import versions_pb2 as tensorflow_dot_core_dot_framework_dot_versions__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/core/protobuf/fingerprint.proto',
  package='tensorflow',
  syntax='proto3',
  serialized_options=_b('\n\030org.tensorflow.frameworkB\021FingerprintProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'),
  serialized_pb=_b('\n*tensorflow/core/protobuf/fingerprint.proto\x12\ntensorflow\x1a(tensorflow/core/framework/versions.proto\"\xcb\x01\n\x0e\x46ingerprintDef\x12\x1a\n\x12graph_def_checksum\x18\x01 \x01(\x04\x12\x1e\n\x16graph_def_program_hash\x18\x02 \x01(\x04\x12\x1a\n\x12signature_def_hash\x18\x03 \x01(\x04\x12\x1f\n\x17saved_object_graph_hash\x18\x04 \x01(\x04\x12\x17\n\x0f\x63heckpoint_hash\x18\x05 \x01(\x04\x12\'\n\x07version\x18\x06 \x01(\x0b\x32\x16.tensorflow.VersionDefB\x89\x01\n\x18org.tensorflow.frameworkB\x11\x46ingerprintProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[tensorflow_dot_core_dot_framework_dot_versions__pb2.DESCRIPTOR,])




_FINGERPRINTDEF = _descriptor.Descriptor(
  name='FingerprintDef',
  full_name='tensorflow.FingerprintDef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='graph_def_checksum', full_name='tensorflow.FingerprintDef.graph_def_checksum', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='graph_def_program_hash', full_name='tensorflow.FingerprintDef.graph_def_program_hash', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature_def_hash', full_name='tensorflow.FingerprintDef.signature_def_hash', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='saved_object_graph_hash', full_name='tensorflow.FingerprintDef.saved_object_graph_hash', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='checkpoint_hash', full_name='tensorflow.FingerprintDef.checkpoint_hash', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='tensorflow.FingerprintDef.version', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=101,
  serialized_end=304,
)

_FINGERPRINTDEF.fields_by_name['version'].message_type = tensorflow_dot_core_dot_framework_dot_versions__pb2._VERSIONDEF
DESCRIPTOR.message_types_by_name['FingerprintDef'] = _FINGERPRINTDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FingerprintDef = _reflection.GeneratedProtocolMessageType('FingerprintDef', (_message.Message,), {
  'DESCRIPTOR' : _FINGERPRINTDEF,
  '__module__' : 'tensorflow.core.protobuf.fingerprint_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.FingerprintDef)
  })
_sym_db.RegisterMessage(FingerprintDef)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
